#!/usr/bin/env python
import os
import shutil
import subprocess
import sys
import time

from charmhelpers.core.hookenv import (
    Hooks,
    UnregisteredHookError,
    status_set,
    log,
    DEBUG,
    INFO,
    service_name,
)
from charmhelpers.fetch import (
    apt_update,
    apt_install,
)

try:
    import netifaces  # NOQA
except ImportError:
    apt_install('python-netifaces')
    import netifaces  # NOQA

try:
    import netaddr  # NOQA
except ImportError:
    apt_install('python-netaddr')
    import netaddr  # NOQA


hooks = Hooks()

INTERFACES = """auto lo
iface lo inet loopback
 post-down modprobe -r i40e

auto eth0
iface eth0 inet manual
# bond-master bond0
# bond-primary eth0

#auto eth1
#iface eth1 inet manual
# bond-master bond0

#auto bond0
#iface bond0 inet manual
# bond-mode active-backup
# bond-miimon 100
# bond-slaves eth0 eth1

# Bridge to use for LXC/KVM containers
auto juju-br0
 iface juju-br0 inet dhcp
 # bridge_ports bond0
 bridge_ports eth0

# oam 3 not used
iface eth2 inet manual

# oam 4 not used
iface eth3 inet manual

# p1p1
auto eth4
iface eth4 inet manual
 bond-master bond1

# p1p2
auto eth5
iface eth5 inet manual
 bond-master bond1

# p2p1
auto eth6
iface eth6 inet manual
 bond-master bond2

# p2p2
auto eth7
iface eth7 inet manual
 bond-master bond2

auto bond1
iface bond1 inet manual
 bond-mode 4
 bond-miimon 100
 bond-lacp-rate 1
 bond-slaves eth4 eth5

auto bond2
iface bond2 inet manual
 bond-mode 4
 bond-miimon 100
 bond-lacp-rate 1
 bond-slaves eth6 eth7

auto brbond1
iface brbond1 inet dhcp
 bridge_ports bond1
 bridge_stp off
 bridge_setageing 0
 bridge_maxwait 10
 post-up /usr/local/bin/setroutes.sh brbond1

auto brbond2
iface brbond2 inet dhcp
 bridge_ports bond2
 bridge_stp off
 bridge_setageing 0
 bridge_maxwait 10
 post-up /usr/local/bin/setroutes.sh brbond2

"""

ROUTE_SCRIPT = r"""#!/usr/bin/env python2
import netifaces
import netaddr
import subprocess
import sys

if __name__ == "__main__":
    iface = sys.argv[1]
    netmap = {18: 31,
              31: 18,
              24: 37,
              37: 24,
              70: 74,
              74: 70}
    routes = []
    for addrs in netifaces.ifaddresses(iface).itervalues():
        for addr in addrs:
            addr = addr['addr']
            try:
                ipver = netaddr.IPAddress(addr)
                if ipver.version == 4:
                    c = addr.split('.')[2]
                    if int(c) in netmap:
                        r = "%s.%s" % ('.'.join(addr.split('.')[:2]),
                                       netmap.get(int(c), c))
                        routes.append("ip r a %s.0/24 via %s.1" % (r, r))
            except netaddr.core.AddrFormatError:
                pass

    for r in routes:
        subprocess.call(r)

"""


@hooks.hook('install')
def install():
    log("Installing", level=DEBUG)

    status_set('maintenance', "configuring machine specific setting")

    # Install required packages
    apt_update()
    apt_install(['vlan', 'ifenslave', 'lldpd'])

    # Set grub, ctrl+alt+del and other details
    with open('/etc/init/control-alt-delete.override', 'w') as fd:
        fd.write('manual\n')

    subprocess.call(r"sed -i '/^\s*GRUB_RECORDFAIL_TIMEOUT\s*=/d' "
                    "/etc/default/grub", shell=True)
    with open('/etc/default/grub', 'a') as fd:
        fd.write('GRUB_RECORDFAIL_TIMEOUT=2\n')

    subprocess.check_call(['update-grub'])

    status_set('maintenance', "reshuffling network configuration")

    GENERATOR = "/lib/udev/rules.d/75-persistent-net-generator.rules"
    RULES = "/etc/udev/rules.d/70-persistent-net.rules"

    # disable persistent-net-generator
    shutil.move(GENERATOR, '/var/tmp/generator.rules')

    pci = []
    out = subprocess.check_output(['lspci', '-m'])
    for l in out.split('\n'):
        splitl = l.split()
        if splitl and "Ethernet controller" in l:
            pci.append(splitl[0])

    with open(RULES, 'w') as fd:
        for i, p in enumerate(pci):
            if i == 0:
                continue

            fd.write('SUBSYSTEM=="net", ACTION=="add", DRIVERS=="?*", '
                     'ATTR{dev_id}=="0x0", ATTR{type}=="1", '
                     'KERNELS=="0000:%s", '
                     'KERNEL=="eth*", NAME="deth%i"\n' % (p, i))

    subprocess.check_call(['udevadm', '--debug', 'control', '--reload-rules'])
    subprocess.check_call(['udevadm', 'trigger', '--action=change',
                           '--attr-match=subsystem=net'])
    subprocess.check_call(['udevadm', 'trigger', '--action=add',
                           '--attr-match=subsystem=net'])

    status_set('maintenance', "network interfaces moved to deth")
    # wait until all interfaces are renamed
    time.sleep(5)

    pci = []
    out = subprocess.check_output(['lspci', '-m'])
    for l in out.split('\n'):
        splitl = l.split()
        if splitl and "Ethernet controller" in l:
            pci.append(splitl[0])

    with open(RULES, 'w') as fd:
        for i, p in enumerate(pci):
            if i == 0:
                continue

            fd.write('SUBSYSTEM=="net", ACTION=="add", DRIVERS=="?*", '
                     'ATTR{dev_id}=="0x0", ATTR{type}=="1", '
                     'KERNELS=="0000:%s", '
                     'KERNEL=="deth*", NAME="eth%i"\n' % (p, i))

    subprocess.check_call(['udevadm', '--debug', 'control', '--reload-rules'])
    subprocess.check_call(['udevadm', 'trigger', '--action=change',
                           '--attr-match=subsystem=net'])
    subprocess.check_call(['udevadm', 'trigger', '--action=add',
                           '--attr-match=subsystem=net'])

    status_set("maintenance", "network interfaces moved to final eth")

    time.sleep(5)

    # re-enable persistent-net-generator
    shutil.move('/var/tmp/generator.rules', GENERATOR)

    status_set("maintenance", "writing /etc/network/interfaces")

    with open('/usr/local/bin/setroutes.py', 'w') as fd:
        fd.write(ROUTE_SCRIPT)

    os.chmod('/usr/local/bin/setroutes.py', 0755)

    with open('/etc/network/interfaces', 'w') as fd:
        fd.write(INTERFACES)

    status_set('maintenance', "ifup all/ reboot instead if unsuccessfull")

    try:
        subprocess.check_call(['ifup', '-a'])
    except subprocess.CalledProcessError:
        log("Interfaces did not come up - rebooting", level=INFO)
        subprocess.check_call(['juju-reboot'])

    status_set('active', "machine configured - network ready")

    if '-api' in service_name() or '-ceph-a' in service_name():
        p = '/usr/local/share/lxc/hooks'
        if not os.path.isdir(p):
            os.makedirs(p)

        p = '/usr/share/lxc/config'
        if not os.path.isdir(p):
            os.makedirs(p)

        f = '/usr/local/share/lxc/hooks/add-more-interfaces'
        if not os.path.isfile(f):
            shutil.copy2('lxc/add-more-interfaces',
                         '/usr/local/share/lxc/hooks/')

        os.chmod('/usr/local/share/lxc/hooks/', 0755)
        target = '/usr/share/lxc/config/ubuntu-cloud.trusty.conf'
        orig = '%s.ubuntu.orig' % target
        subprocess.check_call(['dpkg-divert', '--local', '--divert',
                               orig, '--add', target])

        p = "/usr/share/lxc/config/"
        if not os.path.isdir(p):
            os.makedirs(p)

        with open('/usr/share/lxc/config/ubuntu-cloud.trusty.conf', 'w') as fd:
            fd.write('lxc.hook.clone = '
                     '/usr/local/share/lxc/hooks/add-more-interfaces\n')

        os.chmod('/usr/share/lxc/config/ubuntu-cloud.trusty.conf', 0644)


@hooks.hook('config-changed')
def config_changed():
    log("Configuring", level=DEBUG)


@hooks.hook('upgrade-charm')
def upgrade_charm():
    log("Upgrading", level=DEBUG)
    install()


if __name__ == '__main__':
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log("Unknown hook '%s' - skipping." % (e))
